#define AOQuality 0   //[0 1 2] Increases the quality of Ambient Occlusion from 0 to 2, 0 is default
#define AORadius 1.0 //[0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0] //Changes the radius of Ambient Occlusion to be larger or smaller, 1.0 is default

float dither5x3()
{
	const int ditherPattern[15] = int[15](
		 9, 3, 7,12, 0,
		11, 5, 1,14, 8,
		 2,13,10, 4, 6);

    vec2 position = floor(mod(vec2(texcoord.s * viewWidth,texcoord.t * viewHeight), vec2(5.0,3.0)));

	int dither = ditherPattern[int(position.x) + int(position.y) * 5];

	return float(dither) * 0.0666666666666667f;
}

uniform mat4 gbufferProjection;

float dbao(sampler2D depth){
	float ao = 0.0;
	
	const int aoloop = 3;
	#if AA == 2
	const int aoside = AOQuality+2;
	#else
	const int aoside = 2*AOQuality+4;
	#endif
	float radius = AORadius*0.1/pow(2.0,AOQuality*0.5);
	float dither2 = fract(dither5x3()-dither64);
	float d = ld(texture2D(depth,texcoord.xy).r);
	const float piangle = 0.0174603175;
	float rot = 360/aoside*(dither2+fract(frameCounter/8));
	float size = radius*dither64;
	float sd = 0.0;
	float angle = 0.0;
	float dist = 0.0;
	vec2 scale = vec2(1.0/aspectRatio,1.0) * gbufferProjection[1][1] / (2.74747742 * max(far*d,6.0));
	
	for (int i = 0; i < aoloop; i++) {
		for (int j = 0; j < aoside; j++) {
			sd = ld(texture2D(depth,texcoord.xy+vec2(cos(rot*piangle),sin(rot*piangle)) * size * scale).r);
			float sample = far*(d-sd)/size;
			angle = clamp(0.5-sample,0.0,1.0);
			dist = clamp(0.0625*sample,0.0,1.0);
			sd = ld(texture2D(depth,texcoord.xy-vec2(cos(rot*piangle),sin(rot*piangle)) * size * scale).r);
			sample = far*(d-sd)/size;
			angle += clamp(0.5-sample,0.0,1.0);
			dist += clamp(0.0625*sample,0.0,1.0);
			ao += clamp(angle+dist,0.0,1.0);
			rot += 180.0/aoside;
		}
		rot += 180.0/aoside;
		size += radius*((AOQuality*0.25)*dither64+1.0);
		radius += (AOQuality*0.25)*radius;
		angle = 0.0;
		dist = 0.0;
	}
	
	ao /= aoloop*aoside;
	
	return pow(ao,AOQuality*0.25+1.5);
}